program boson

! *******************************************************************
! *                                                                 *
! * perform the quantum-mechanical complement to the classical step *
! * of the dielectric theory of eels in specular geometry using a   *
! * suitable thermodynamical average of the quantized surface       *
! * harmonic oscillators                                            *
! *                                                                 *
! *******************************************************************

  implicit none

  integer, parameter :: mmax=14, nmax=2**mmax
  double precision :: asym, emax, emin, gauss, t, width, wmin, wmax, y
  double precision :: p2(nmax)
  integer :: i, np, ioStatus
  character (len = 72) :: comment(2)
  double precision, allocatable :: xout(:), yout(:)
  integer :: nout

! *** read input parameters

      call FCAT_boson_all(1)
  call change_working_dir()
      call FCAT_boson_all(2)
  open(unit = 13, file = 'bosin')
! target temperature (Kelvin)
      call FCAT_boson_all(3)
  read(13, *) t
! width of the instrumental response (cm**-1)
      call FCAT_boson_all(4)
  read(13, *) width
! fraction of gaussian for the instrumental response
      call FCAT_boson_all(5)
  read(13, *) gauss
! asymmetry of the instrumental response
      call FCAT_boson_all(6)
  read(13, *) asym
! lower and upper energy losses for this computation (cm**-1)
      call FCAT_boson_all(7)
  read(13, *) emin
      call FCAT_boson_all(8)
  read(13, *) emax
      call FCAT_boson_all(9)
  close(unit=13)

      call FCAT_boson_all(10)
  write(*,*) 'program boson (September 2020)'
      call FCAT_boson_all(11)
  write(*,'(a, f6.1, a, f7.2, a)') ' t =', t, ' K, width =', width, ' cm**-1'
      call FCAT_boson_all(12)
  write(*,'(a, f5.2, a, f5.2)') ' gauss =', gauss, ', asym =', asym
      call FCAT_boson_all(13)
  write(*,'(a, g11.4, a, g11.4, a)') ' energy losses from', emin, ' to', emax, ' cm**-1'

! *** read the table of values of the classical loss spectrum

      call FCAT_boson_all(14)
  open(unit = 12, file = 'eelsou')
      call FCAT_boson_all(15)
  read(12, '(a48)') comment(1)
      call FCAT_boson_all(16)
  read(12, '(a72)') comment(2)
      call FCAT_boson_all(17)
  np = 0
      call FCAT_boson_all(18)
  do
      call FCAT_boson_all(19)
    read(12, *, IOSTAT = ioStatus) wmax, y
      call FCAT_boson_all(20)
    if (ioStatus /= 0) exit
      call FCAT_boson_all(21)
    if (wmax < 0.0d0) cycle
      call FCAT_boson_all(22)
    np = np + 1
      call FCAT_boson_all(23)
    p2(np) = y
      call FCAT_boson_all(24)
    if (np == 1) wmin = wmax
      call FCAT_boson_all(25)
    if (np < nmax) cycle
      call FCAT_boson_all(26)
  enddo
      call FCAT_boson_all(27)
  close(unit = 12)

      call FCAT_boson_all(28)
  if (np <= 0) then 
      call FCAT_boson_all_rep()
       stop '*** no input values for pcl ***'
       endif

      call FCAT_boson_all(29)
  write(*,*) comment(2)
      call FCAT_boson_all(30)
  write(*,'(a, i6, a, g15.7, a, g15.7)') ' read', np, ' values of pcl from', wmin, ' to', wmax

! length calculation for xout, yout.
!
      call FCAT_boson_all(31)
  nout = 2**14
      call FCAT_boson_all(32)
  allocate (xout(nout))
      call FCAT_boson_all(33)
  allocate (yout(nout))

      call FCAT_boson_all(34)
  call doboson(t, width, gauss, asym, emin, emax, wmin, wmax, np, p2, xout, yout, nout)

      call FCAT_boson_all(35)
  open(unit = 14, file = 'bosou')
      call FCAT_boson_all(36)
  write(14, '(a, a, f6.1, a, f5.2)') comment(1), 'T =', t, ' GAUSS =', gauss
      call FCAT_boson_all(37)
  write(14, *) comment(2)
      call FCAT_boson_all(38)
  do i = 1, nout
      call FCAT_boson_all(39)
    write(14, '(2e15.7)') xout(i), yout(i)
      call FCAT_boson_all(40)
  end do
      call FCAT_boson_all(41)
  close(unit = 14)
      call FCAT_boson_all(42)
  write(*,*) nout, ' values written on disk'

      call FCAT_boson_all(43)
  deallocate (xout, yout)
      call FCAT_boson_all(44)
      call FCAT_boson_all_rep()
  stop
      call FCAT_boson_all_rep()
end program boson

subroutine change_working_dir()

! This routine gets the first argument of the commandline and takes it
! as the path to change the working directory
! used intrinsic routines:
! iarg returns the number of commandline arguments without the program cname.
! chdir changes the directory and returns 0 on success.
! trim removes blanks from strings.

  character (len = 256) :: argument
  integer :: status

      call FCAT_boson_all(45)
  if (iargc() == 1) then
      call FCAT_boson_all(46)
    call getarg(1, argument)
      call FCAT_boson_all(47)
    status = chdir(trim(argument))
      call FCAT_boson_all(48)
    if (status /= 0) then
      call FCAT_boson_all(49)
      write (*,*) '*** change directory failed ***'
      call FCAT_boson_all(50)
      write (*,*) 'directory tried: ', trim(argument)
      call FCAT_boson_all(51)
      write (*,*) 'error code (see: man chdir): ', status
      call FCAT_boson_all(52)
      write (*,*) 'continuing in the start directory'
      call FCAT_boson_all(53)
    end if
      call FCAT_boson_all(54)
  end if

      call FCAT_boson_all(55)
  return
end subroutine change_working_dir

double precision function respon(w, width)

!*******************************************************************
!                                                                  *
! instrumental response of the spectrometer for the frequency w    *
! width is the full width at half maximum of the response function *
!                                                                  *
!*******************************************************************

  implicit none
  double precision, intent(in) :: w, width

  double precision :: u

      call FCAT_boson_all(56)
  u = 1.25d0 * w / width
      call FCAT_boson_all(57)
  if (u <= -1.0d0 .or. u >= 2.0d0) then
      call FCAT_boson_all(58)
     respon = 0.0d0
  elseif (u > 1.0d0) then
      call FCAT_boson_all(59)
      call FCAT_boson_all(60)
     respon = 4.0d0 + 2 * dsqrt((u - 1.0d0)**3) - 3 * u
  elseif (u > 0.0d0) then
      call FCAT_boson_all(61)
      call FCAT_boson_all(62)
     respon = 2.0d0 - 4 * dsqrt(u**3) + 3 * u
  else
      call FCAT_boson_all(63)
      call FCAT_boson_all(64)
     respon = 2 * dsqrt((u + 1.0d0)**3)
      call FCAT_boson_all(65)
  endif
      call FCAT_boson_all(66)
  return
      call FCAT_boson_all_rep()
end function respon

subroutine sicot(f, m, h, x0)

! *******************************************************************
! *                                                                 *
! * integral transform g(y, x0) of a real function f(x) defined by  *
! *                                                                 *
! * g(x0, y) = integral of f(x)/tanh(x/x0)*(1-cos(x*y)) dx          *
! *                                                                 *
! * f(x) is tabulated on the mesh of points xj = (j-1/2)*h ,        *
! * j = 1, 2, ..., n ,  with n = 2**m                               *
! * g(x0, y) is computed on the mesh of points yk = k*2pi/(n*h) ,   *
! * k = 0, 1, 2, ..., n-1                                           *
! *                                                                 *
! * input ...                                                       *
! * f(j) contains the value of f(xj) , j = 1, 2, ..., n             *
! * m is such that n = 2**iabs(m)                                   *
! * h is the step size (must be positive)                           *
! *                                                                 *
! * output ...                                                      *
! * f(k+1) contains g(x0, yk), k = 0, 1, ..., n-1                   *
! *                                                                 *
! * computing remarks ...                                           *
! * f is a real array with dimension n or more                      *
! * it is supposed that f(x) is zero for x > n*h                    *
! * external reference : sintr (sine transform of a real function). *
! *                                                                 *
! *******************************************************************

  implicit none

  double precision, intent(in out) :: f(*)
  integer, intent(in) :: m
  double precision, intent(in) :: h
  double precision, intent(in) :: x0

  double precision :: a, b, c, d, e, s, sa
  integer :: i, j, msign, n

  logical :: debug

  interface
    subroutine sintr(f, msign, h)
      double precision, intent(in out) :: f(*)
      integer, intent(in) :: msign
      double precision, intent(in) :: h
    end subroutine sintr
  end interface  

      call FCAT_boson_all(67)
  debug = .false.
      call FCAT_boson_all(68)
  if (m == 0) then
      call FCAT_boson_all(69)
    f(1) = 0.0d0
      call FCAT_boson_all(70)
    return
      call FCAT_boson_all(71)
  endif

      call FCAT_boson_all(72)
  if (h <= 0.0d0) then
      call FCAT_boson_all(73)
    if (debug) write(*,*) ' *** incorrect step size in <sicot>, h =', h, ' ***'
      call FCAT_boson_all(74)
      call FCAT_boson_all_rep()
    stop
      call FCAT_boson_all(75)
  endif

      call FCAT_boson_all(76)
  if (x0 < 0.0d0) then
      call FCAT_boson_all(77)
    if (debug) write(*,*) ' *** incorrect input in <sicot>, x0 =', x0, ' ***'
      call FCAT_boson_all(78)
      call FCAT_boson_all_rep()
    stop
      call FCAT_boson_all(79)
  endif

      call FCAT_boson_all(80)
  n = 2**iabs(m)

! *** evaluate the integral from xj to infinity of f(x)/tanh(x/x0) dx
! and store the result in f(j)

      call FCAT_boson_all(81)
  c = 0.0d0
      call FCAT_boson_all(82)
  if (x0 > h / 16) then
      call FCAT_boson_all(83)
    c = dexp(-h / x0)
      call FCAT_boson_all(84)
  endif
      call FCAT_boson_all(85)
  s = 1.0d0 - c
      call FCAT_boson_all(86)
  e = c**2
      call FCAT_boson_all(87)
  a = 0.25d0 * h
      call FCAT_boson_all(88)
  b = 0.25d0 * x0
      call FCAT_boson_all(89)
  do i = 1, n
      call FCAT_boson_all(90)
    if (i < n) then
      call FCAT_boson_all(91)
      f(i) = f(i) + f(i+1)
      call FCAT_boson_all(92)
    endif
      call FCAT_boson_all(93)
    d = a
      call FCAT_boson_all(94)
    if (s /= 1.0d0) then
      call FCAT_boson_all(95)
      sa = s
      call FCAT_boson_all(96)
      c = c * e
      call FCAT_boson_all(97)
      s = 1.0d0 - c
      call FCAT_boson_all(98)
      d = d + b * dlog(s / sa)
      call FCAT_boson_all(99)
    endif
      call FCAT_boson_all(100)
    f(i) = f(i) * d
      call FCAT_boson_all(101)
  enddo

      call FCAT_boson_all(102)
  j = n
      call FCAT_boson_all(103)
  do i = 2, n
      call FCAT_boson_all(104)
    j = j - 1
      call FCAT_boson_all(105)
    f(j) = f(j) + f(j + 1)
      call FCAT_boson_all(106)
  enddo
! alternative, but not yet tested:
!  do j = n - 1, 1, -1
!    f(j) = f(j) + f(j + 1)
!  enddo
      call FCAT_boson_all(107)
  msign = -iabs(m)
      call FCAT_boson_all(108)
  call sintr(f, msign, h)
      call FCAT_boson_all(109)
  return
end subroutine sicot

subroutine sintr(f, msign, h)

! *******************************************************************
! *                                                                 *
! * integral sine transform of a real function f(x)                 *
! *                                                                 *
! * g(y) = integral from zero to infinity of f(x)*sin(x*y) dx       *
! * if msign >= 0 or                                                *
! * g(y) = y * integral from zero to infinity of f(x)*sin(x*y) dx   *
! * if msign < 0                                                    *
! *                                                                 *
! * f(x) is tabulated on the mesh of points xj = (j-1/2)*h ,        *
! * j = 1, 2, ..., n ,  with n = 2**iabs(msig)                      *
! * g(y) is computed on the mesh of points yk = k*2pi/(n*h) ,       *
! * k = 0, 1, 2, ..., n-1                                           *
! *                                                                 *
! * input ...                                                       *
! * f(j) contains the value of f(xj) , j = 1, 2, ..., n             *
! * msign is such that n = 2**iabs(msign)                           *
! * h is the step size (must be positive)                           *
! *                                                                 *
! * output ...                                                      *
! * f(k+1) contains g(yk), k = 0, 1, ..., n-1                       *
! *                                                                 *
! * computing remarks ...                                           *
! * f is a real array with dimension n or more                      *
! * it is supposed that f(x) is zero for x > n*h                    *
! *                                                                 *
! *******************************************************************

  implicit none

  double precision, intent(in out) :: f(*)
  integer, intent(in) :: msign
  double precision, intent(in) :: h

  double precision :: a, c, ca, d, e, fnp1, s, sa, ti, ti1, ti2, tr, tr1, tr2
  double precision :: ui, ur, wi, wr
  integer :: i, i2, ip2, j, j2, k, l, le, le1, m2, n, n2, nm1, nv2

  logical :: debug

      call FCAT_boson_all(110)
  debug = .false.
      call FCAT_boson_all(111)
  if (h <= 0.0d0) then
      call FCAT_boson_all(112)
    if (debug) write(*,*) ' *** incorrect step size in <sintr>, h: ', h, ' ***'
      call FCAT_boson_all(113)
      call FCAT_boson_all_rep()
    stop
      call FCAT_boson_all(114)
  endif

      call FCAT_boson_all(115)
  if (msign == 0) then
      call FCAT_boson_all(116)
    f(1) = 0.0d0
      call FCAT_boson_all(117)
    return
      call FCAT_boson_all(118)
  endif

! *** for j and k = 0, 1, 2 ... n/2-1, compute
! s1 = sum ( f(2*j+1)*cos(4*pi*k*j/n) - f(2*j+2)*sin(4*pi*k*j/n) )
! s2 = sum ( f(2*j+1)*sin(4*pi*k*j/n) + f(2*j+2)*cos(4*pi*k*j/n) )
! store s1 in f(2*k+1) and s2 in f(2*k+2)
! (adapted from cfft routine, cern library member d704)

      call FCAT_boson_all(119)
  m2 = iabs(msign) - 1
      call FCAT_boson_all(120)
  n = 2**m2
      call FCAT_boson_all(121)
  if (n /= 1) then
      call FCAT_boson_all(122)
    nv2 = n / 2
      call FCAT_boson_all(123)
    nm1 = n - 1
      call FCAT_boson_all(124)
    j = 1
      call FCAT_boson_all(125)
    do i = 1, nm1
      call FCAT_boson_all(126)
      if (i < j) then
      call FCAT_boson_all(127)
        i2 = i + i
      call FCAT_boson_all(128)
        j2 = j + j
      call FCAT_boson_all(129)
        ti = f(j2)
      call FCAT_boson_all(130)
        f(j2) = f(i2)
      call FCAT_boson_all(131)
        f(i2) = ti
      call FCAT_boson_all(132)
        i2 = i2 - 1
      call FCAT_boson_all(133)
        j2 = j2 - 1
      call FCAT_boson_all(134)
        tr = f(j2)
      call FCAT_boson_all(135)
        f(j2) = f(i2)
      call FCAT_boson_all(136)
        f(i2) = tr
      call FCAT_boson_all(137)
      endif
      call FCAT_boson_all(138)
      k = nv2
      call FCAT_boson_all(139)
      do while (j > k)
      call FCAT_boson_all(140)
        j = j - k
      call FCAT_boson_all(141)
        k = k / 2
      call FCAT_boson_all(142)
      enddo
      call FCAT_boson_all(143)
      j = j + k
      call FCAT_boson_all(144)
    enddo
      call FCAT_boson_all(145)
    do i = 1, n, 2
      call FCAT_boson_all(146)
      i2 = i + i
      call FCAT_boson_all(147)
      ti = f(i2 + 2)
      call FCAT_boson_all(148)
      f(i2 + 2) = f(i2) - ti
      call FCAT_boson_all(149)
      f(i2) = f(i2) + ti
      call FCAT_boson_all(150)
      i2 = i2 - 1
      call FCAT_boson_all(151)
      tr = f(i2 + 2)
      call FCAT_boson_all(152)
      f(i2 + 2) = f(i2) - tr
      call FCAT_boson_all(153)
      f(i2) = f(i2) + tr
      call FCAT_boson_all(154)
    enddo
      call FCAT_boson_all(155)
    if (m2 /= 1) then
      call FCAT_boson_all(156)
      c = 0.0d0
      call FCAT_boson_all(157)
      s = 1.0d0
      call FCAT_boson_all(158)
      le = 2
      call FCAT_boson_all(159)
      do l = 2, m2
      call FCAT_boson_all(160)
        wr = c
      call FCAT_boson_all(161)
        wi = s
      call FCAT_boson_all(162)
        ur = wr
      call FCAT_boson_all(163)
        ui = wi
      call FCAT_boson_all(164)
        c = dsqrt(c * 0.5d0 + 0.5d0 )
      call FCAT_boson_all(165)
        s = s / (c + c)
      call FCAT_boson_all(166)
        le1 = le
      call FCAT_boson_all(167)
        le = le1 + le1
      call FCAT_boson_all(168)
        do i = 1, n, le
      call FCAT_boson_all(169)
          i2 = i + i
      call FCAT_boson_all(170)
          ip2 = i2 + le
      call FCAT_boson_all(171)
          ti = f(ip2)
      call FCAT_boson_all(172)
          f(ip2) = f(i2) - ti
      call FCAT_boson_all(173)
          f(i2) = f(i2) + ti
      call FCAT_boson_all(174)
          i2 = i2 - 1
      call FCAT_boson_all(175)
          ip2 = ip2 - 1
      call FCAT_boson_all(176)
          tr = f(ip2)
      call FCAT_boson_all(177)
          f(ip2) = f(i2) - tr
      call FCAT_boson_all(178)
          f(i2) = f(i2) + tr
      call FCAT_boson_all(179)
        enddo
      call FCAT_boson_all(180)
        do j = 2, le1
      call FCAT_boson_all(181)
          do i = j, n, le
      call FCAT_boson_all(182)
            i2 = i + i
      call FCAT_boson_all(183)
            ip2 = i2 + le
      call FCAT_boson_all(184)
            tr = f(ip2 - 1) * ur - f(ip2) * ui
      call FCAT_boson_all(185)
            ti = f(ip2) * ur + f(ip2 - 1) * ui
      call FCAT_boson_all(186)
            f(ip2) = f(i2) - ti
      call FCAT_boson_all(187)
            f(i2) = f(i2) + ti
      call FCAT_boson_all(188)
            i2 = i2 - 1
      call FCAT_boson_all(189)
            ip2 = ip2 - 1
      call FCAT_boson_all(190)
            f(ip2) = f(i2) - tr
      call FCAT_boson_all(191)
            f(i2) = f(i2) + tr
      call FCAT_boson_all(192)
          enddo
      call FCAT_boson_all(193)
          tr = ur * wr - ui * wi
      call FCAT_boson_all(194)
          ui = ui * wr + ur * wi
      call FCAT_boson_all(195)
          ur = tr
      call FCAT_boson_all(196)
        enddo
      call FCAT_boson_all(197)
      enddo
      call FCAT_boson_all(198)
    endif
      call FCAT_boson_all(199)
  endif

! *** for j and k = 0, 1 ... n-1, transform the array f so obtained into
! sum f(j+1)*cos(2*pi*k*j/n) and sum f(j+1)*sin(2*pi*k*j/n)
! and multiply the results by (1 - cos(2*pi*k/n)) and sin(2*pi*k/n)

      call FCAT_boson_all(200)
  fnp1 = 4 * (f(1) - f(2))
      call FCAT_boson_all(201)
  a = 3.141592653589793238d0 / n
      call FCAT_boson_all(202)
  n2 = n + 1
      call FCAT_boson_all(203)
  n = 2 * n
      call FCAT_boson_all(204)
  if (n2 /= 2) then
      call FCAT_boson_all(205)
    c = 1.0d0
      call FCAT_boson_all(206)
    s = 0.0d0
      call FCAT_boson_all(207)
    ca = dcos(a)
      call FCAT_boson_all(208)
    sa = dsin(a)
      call FCAT_boson_all(209)
    k = n + 1
      call FCAT_boson_all(210)
    do j = 3, n2, 2
      call FCAT_boson_all(211)
      k = k - 2
      call FCAT_boson_all(212)
      d = c
      call FCAT_boson_all(213)
      c = d * ca - s * sa
      call FCAT_boson_all(214)
      s = d * sa + s * ca
      call FCAT_boson_all(215)
      tr1 = f(j) + f(k)
      call FCAT_boson_all(216)
      ti1 = f(j + 1) - f(k + 1)
      call FCAT_boson_all(217)
      d = f(j) - f(k)
      call FCAT_boson_all(218)
      e = f(j + 1) + f(k + 1)
      call FCAT_boson_all(219)
      tr2 = d * s + e * c
      call FCAT_boson_all(220)
      ti2 = e * s - d * c
      call FCAT_boson_all(221)
      f(j) = (1.0d0 - c) * (tr1 + tr2)
      call FCAT_boson_all(222)
      f(j + 1) = s * (ti1 + ti2)
      call FCAT_boson_all(223)
      f(k) = (1.0d0 + c) * (tr1 - tr2)
      call FCAT_boson_all(224)
      f(k + 1) = s * (ti2 - ti1)
      call FCAT_boson_all(225)
    enddo
      call FCAT_boson_all(226)
    n2 = n2 - 1
      call FCAT_boson_all(227)
    do j = 2, n2
      call FCAT_boson_all(228)
      j2 = j + j
      call FCAT_boson_all(229)
      f(j) = f(j2 - 1) + f(j2)
      call FCAT_boson_all(230)
    enddo
      call FCAT_boson_all(231)
    do j = 2, n2
      call FCAT_boson_all(232)
      f(n + 2 - j) = f(j)
      call FCAT_boson_all(233)
    enddo
      call FCAT_boson_all(234)
    n2 = n2 + 1
      call FCAT_boson_all(235)
  endif
      call FCAT_boson_all(236)
  f(n2) = fnp1
      call FCAT_boson_all(237)
  if (msign >= 0) then

! *** normalization

      call FCAT_boson_all(238)
    c = 0.0d0
      call FCAT_boson_all(239)
    a = (a + a) / h
      call FCAT_boson_all(240)
    do j = 2, n
      call FCAT_boson_all(241)
      c = c + a
      call FCAT_boson_all(242)
      f(j) = f(j) / c
      call FCAT_boson_all(243)
    enddo
      call FCAT_boson_all(244)
  endif
      call FCAT_boson_all(245)
  f(1) = 0.0d0
      call FCAT_boson_all(246)
  return
end subroutine sintr

subroutine rcffi(ar, ai, msign, h)

!   *******************************************************************
!   *                                                                 *
!   * using a radix-two fast-fourier-transform technique, rcffi       *
!   * computes the fourier integral transform of a real function      *
!   * f(x) or the inverse fourier transform of a complex function     *
!   * such that g(-y) = conj(g(y)).                                   *
!   * (adapted from cfft routine, cern library member d704)           *
!   *                                                                 *
!   * g(y) = integral f(x) * cexp(-i * x * y) dx    (msign < 0)       *
!   *                                                                 *
!   * f(x) = 1 / (2 * pi) integral g(y) * cexp(+i * y * x) dy  (msign > 0)    *
!   *                                                                 *
!   * f(x) is tabulated on the meshes of points defined by            *
!   * xj = j * hx and -xj = -j * hx,  j = 0, +1, ..., n-2, n-1        *
!   * ar(j+1) contains f(+xj) and ai(j+1) contains f(-xj)             *
!   * (input when msign < 0 , output when msign >0)                   *
!   * ar(1) may be different from ai(1)                               *
!   *                                                                 *
!   * g(y) is tabulated on the mesh of points defined by              *
!   * yk = k * hy  , k = 0, 1, ..., n - 2, n - 1                      *
!   * ar(j+1) contains real(g(yk)) and ai(j+1) contains aimag(g(yk))  *
!   * (output when msign > 0 , input when msign < 0)                  *
!   * remark :  g(-yk) = conj(g(+yk))                                 *
!   *                                                                 *
!   * n = 2**iabs(msign)  (msign is an input)                         *
!   *                                                                 *
!   * the step sizes satisfy                                          *
!   * hy = (2 * pi) / (n * hx)   or    hx = (2 * pi) / (n * hy)       *
!   * h is an input (h = hx if msign < 0 , h = hy if msign > 0)       *
!   * h must be positive                                              *
!   *                                                                 *
!   * ar and ai are two real arrays with dimension n (input and       *
!   * output)                                                         *
!   *                                                                 *
!   *******************************************************************

  implicit none

  double precision, intent(in out) :: ar(*)
  double precision, intent(in out) :: ai(*)
  integer, intent(in)  :: msign
  double precision, intent(in) :: h

  double precision :: a, as, c, s, t, ti, tr, ui, ur, wi, wr
  integer :: i, ip, j, k, l, le, le1, m, n, nm1, nv2
  logical :: debug

      call FCAT_boson_all(247)
  debug = .false.
      call FCAT_boson_all(248)
  as = 0.0d0
      call FCAT_boson_all(249)
  if (msign == 0) then
      call FCAT_boson_all(250)
    return
      call FCAT_boson_all(251)
  endif

      call FCAT_boson_all(252)
  if (h <= 0.0d0) then
      call FCAT_boson_all(253)
    if (debug) write(*,*) ' *** negative step size in <rcffi>, h = ', h, ' *** '
      call FCAT_boson_all(254)
      call FCAT_boson_all_rep()
    stop
      call FCAT_boson_all(255)
  endif

! *** initialization

      call FCAT_boson_all(256)
  m = iabs(msign)
      call FCAT_boson_all(257)
  n = 2**m
      call FCAT_boson_all(258)
  if (msign > 0) then
      call FCAT_boson_all(259)
    ar(1) = 0.5d0 * ar(1)
  else
      call FCAT_boson_all(260)
      call FCAT_boson_all(261)
    as = ar(1) - ai(1)
      call FCAT_boson_all(262)
    ar(1) = 0.5d0 * (ar(1) + ai(1))
      call FCAT_boson_all(263)
    do i = 2, n
      call FCAT_boson_all(264)
      ar(i) = ar(i) + ai(n - i + 2)
      call FCAT_boson_all(265)
    enddo
      call FCAT_boson_all(266)
    do i = 1, n
      call FCAT_boson_all(267)
      ai(i) = 0.0d0
      call FCAT_boson_all(268)
    enddo
      call FCAT_boson_all(269)
  endif
! *** discrete fast-fourier transform
      call FCAT_boson_all(270)
  nv2 = n / 2
      call FCAT_boson_all(271)
  nm1 = n - 1
      call FCAT_boson_all(272)
  j = 1
      call FCAT_boson_all(273)
  do i = 1, nm1
      call FCAT_boson_all(274)
    if (i < j) then
      call FCAT_boson_all(275)
      tr = ar(j)
      call FCAT_boson_all(276)
      ar(j) = ar(i)
      call FCAT_boson_all(277)
      ar(i) = tr
      call FCAT_boson_all(278)
      ti = ai(j)
      call FCAT_boson_all(279)
      ai(j) = ai(i)
      call FCAT_boson_all(280)
      ai(i) = ti
      call FCAT_boson_all(281)
    endif
      call FCAT_boson_all(282)
    k = nv2
      call FCAT_boson_all(283)
    do while (j > k)
      call FCAT_boson_all(284)
      j = j - k
      call FCAT_boson_all(285)
      k = k / 2
      call FCAT_boson_all(286)
    enddo
      call FCAT_boson_all(287)
    j = j + k
      call FCAT_boson_all(288)
  enddo
      call FCAT_boson_all(289)
  do i = 1, n, 2
      call FCAT_boson_all(290)
    tr = ar(i + 1)
      call FCAT_boson_all(291)
    ar(i + 1) = ar(i) - tr
      call FCAT_boson_all(292)
    ar(i) = ar(i) + tr
      call FCAT_boson_all(293)
    ti = ai(i + 1)
      call FCAT_boson_all(294)
    ai(i + 1) = ai(i) - ti
      call FCAT_boson_all(295)
    ai(i) = ai(i) + ti
      call FCAT_boson_all(296)
  enddo
      call FCAT_boson_all(297)
  if (m == 1) return
      call FCAT_boson_all(298)
  c = 0.0d0
      call FCAT_boson_all(299)
  s = dble(isign(1, msign))
      call FCAT_boson_all(300)
  le = 2
      call FCAT_boson_all(301)
  do l = 2, m
      call FCAT_boson_all(302)
    wr = c
      call FCAT_boson_all(303)
    ur = wr
      call FCAT_boson_all(304)
    wi = s
      call FCAT_boson_all(305)
    ui = wi
      call FCAT_boson_all(306)
    c = dsqrt(c * 0.5d0 + 0.5d0)
      call FCAT_boson_all(307)
    s = wi / (c + c)
      call FCAT_boson_all(308)
    le1 = le
      call FCAT_boson_all(309)
    le = le1 + le1
      call FCAT_boson_all(310)
    do i = 1, n, le
      call FCAT_boson_all(311)
      ip = i + le1
      call FCAT_boson_all(312)
      tr = ar(ip)
      call FCAT_boson_all(313)
      ar(ip) = ar(i) - tr
      call FCAT_boson_all(314)
      ar(i)  = ar(i) + tr
      call FCAT_boson_all(315)
      ti = ai(ip)
      call FCAT_boson_all(316)
      ai(ip) = ai(i) - ti
      call FCAT_boson_all(317)
      ai(i)  = ai(i) + ti
      call FCAT_boson_all(318)
    enddo
      call FCAT_boson_all(319)
    do j = 2, le1
      call FCAT_boson_all(320)
      do i = j, n, le
      call FCAT_boson_all(321)
        ip = i + le1
      call FCAT_boson_all(322)
        tr = ar(ip) * ur - ai(ip) * ui
      call FCAT_boson_all(323)
        ti = ar(ip) * ui + ai(ip) * ur
      call FCAT_boson_all(324)
        ar(ip) = ar(i) - tr
      call FCAT_boson_all(325)
        ar(i)  = ar(i) + tr
      call FCAT_boson_all(326)
        ai(ip) = ai(i) - ti
      call FCAT_boson_all(327)
        ai(i)  = ai(i) + ti
      call FCAT_boson_all(328)
      enddo
      call FCAT_boson_all(329)
      tr = ur
      call FCAT_boson_all(330)
      ur = tr * wr - ui * wi
      call FCAT_boson_all(331)
      ui = tr * wi + ui * wr
      call FCAT_boson_all(332)
    enddo
      call FCAT_boson_all(333)
  enddo

! *** correction of the discrete fft, using a trapezoidal approximation
! *** for the variations of the input function

      call FCAT_boson_all(334)
  c = h
      call FCAT_boson_all(335)
  if (msign >= 0) then
      call FCAT_boson_all(336)
    c = c / 3.141592653589793238d0
      call FCAT_boson_all(337)
    ai(1) = ar(1)
      call FCAT_boson_all(338)
    do i = 2, n
      call FCAT_boson_all(339)
      ai(i) = ar(n - i + 2)
      call FCAT_boson_all(340)
    enddo
      call FCAT_boson_all(341)
  endif
      call FCAT_boson_all(342)
  ar(1) = c * ar(1)
      call FCAT_boson_all(343)
  ai(1) = c * ai(1)
      call FCAT_boson_all(344)
  a = 3.141592653589793238d0 / n
      call FCAT_boson_all(345)
  wr = dcos(a)
      call FCAT_boson_all(346)
  wi = dsin(a)
      call FCAT_boson_all(347)
  t = dsqrt(c)
      call FCAT_boson_all(348)
  a = a / t
      call FCAT_boson_all(349)
  c = 0.0d0
      call FCAT_boson_all(350)
  ur = 1.0d0
      call FCAT_boson_all(351)
  ui = 0.0d0
      call FCAT_boson_all(352)
  do i = 2, n
      call FCAT_boson_all(353)
    c = c + a
      call FCAT_boson_all(354)
    tr = ur
      call FCAT_boson_all(355)
    ur = wr * tr - wi * ui
      call FCAT_boson_all(356)
    ui = wi * tr + wr * ui
      call FCAT_boson_all(357)
    ti = ui / c
      call FCAT_boson_all(358)
    tr = ti**2
      call FCAT_boson_all(359)
    ar(i) = ar(i) * tr
      call FCAT_boson_all(360)
    ai(i) = ai(i) * tr
      call FCAT_boson_all(361)
    if (msign <= 0) then
      call FCAT_boson_all(362)
      if (as /= 0.0d0) then
      call FCAT_boson_all(363)
        ai(i) = ai(i) - as * (t - ur * ti) / (2 * c)
      call FCAT_boson_all(364)
      endif
      call FCAT_boson_all(365)
    endif
      call FCAT_boson_all(366)
  enddo
      call FCAT_boson_all(367)
  return
end subroutine rcffi

subroutine doboson(t, width, gauss, asym, emin, emax, wmin, wmax, np, p, xout, yout, nout)

! *******************************************************************
! *                                                                 *
! * perform the quantum-mechanical complement to the classical step *
! * of the dielectric theory of eels in specular geometry using a   *
! * suitable thermodynamical average of the quantized surface       *
! * harmonic oscillators                                            *
! *                                                                 *
! * t:      Target temperature                                      *
! * width:  Width of instrumental response (cm**-1)                 *
! * gauss:  Fraction of gaussian for the instrumental response      *
! * asym:   Asymmetry of the instrumental response                  *
! * emin:   Lower loss energy for this computation (cm**-1)         *
! * emax:   Upper loss energy for this computation (cm**-1)         *
! * wmin:   Lower loss energy of the eels computation (cm**-1)      *
! * wmax:   Upper loss energy of the eels computation (cm**-1)      *
! * np:     Number of points of the eels computation                *
! * p:      Intensities of the eels computation                     *
! * xout:   Energies of this computation                            *
! * yout:   Intensities of this computation                         *
! * nout:   Number of points of this computation                    *
! *******************************************************************

  implicit none

  double precision, intent(in) :: t, width, gauss, asym, emin, emax, wmin
  double precision, intent(in out) :: wmax
  integer, intent(in) :: np
  double precision, intent(in) :: p(np)
  double precision, intent(in out) :: xout(nout), yout(nout)
  integer, intent(in out) :: nout

  integer, parameter :: mmax = 14, nmax = 2**mmax
  double precision :: a, a1, a2, alfa, anorm, b, cp2, dwn, fac, fi
  double precision :: fm, fm0, fm1, fp1, fmpic, fp, fp0, fppic, fr, g1, g2
  double precision :: h, pi, sigma, sp2, test, u
  double precision :: wmpic, wppic, wn, x, x1, x2, x3
  double precision :: o1, o2, respon

  external o1, o2

  integer :: i, imax, imin, istep, j, jmin, jmax, m, n
  logical :: picm, picp
  double precision, allocatable :: p1(:), p2(:)
  double precision :: r1(nmax), r2(nmax)

  logical :: debug

  interface
    subroutine rcffi(ar, ai, msign, h)
      double precision, intent(in out) :: ar(*)
      double precision, intent(in out) :: ai(*)
      integer, intent(in)  :: msign
      double precision, intent(in) :: h
    end subroutine rcffi
    subroutine sicot(f, m, h, x0)
      double precision, intent(in out) :: f(*)
      integer, intent(in) :: m
      double precision, intent(in) :: h
      double precision, intent(in) :: x0
    end subroutine sicot
    subroutine sintr(f, msign, h)
      double precision, intent(in out) :: f(*)
      integer, intent(in) :: msign
      double precision, intent(in) :: h
    end subroutine sintr
  end interface  

! remark : the two arrays r1 and r2 are used for fourier
! transforming the user-supplied instrumental response of the
! spectrometer that has to be coded in the external routine
! respon called when the input parameter gauss is < 0 or > 1.
! with 0 <= gauss <= 1, r1 and r2 are not used.

! *** rational approximations for ei(u) * exp(-u) + e1(u) * exp(+u)
! *** in the intervals (0, 1.3) and (1.3, infinity) <accuracy : 4.e-04>
! *** used for fourier transforming half-lorentzian functions


      call FCAT_boson_all(368)
  debug = .false.
      call FCAT_boson_all(369)
  data fm1 / 0.0d0 /, fp1 / 0.0d0 /
      call FCAT_boson_all(370)
  pi = 4 * datan(1.0d0)

      call FCAT_boson_all(371)
  dwn = (wmax - wmin) / (np - 1)

! *** redefine the frequency interval (0, wmax) to be used for the
! *** fourier transforms

      call FCAT_boson_all(372)
  if (debug) write(*,*) 'wmin: ', wmin, 'wmax: ', wmax, 'dwn: ', dwn
      call FCAT_boson_all(373)
  jmin = int(0.5d0 + wmin / dwn)
      call FCAT_boson_all(374)
  if ((jmin - 0.5d0) * dwn < wmin) then
      call FCAT_boson_all(375)
    jmin = jmin + 1
      call FCAT_boson_all(376)
  endif
      call FCAT_boson_all(377)
  fac = ((jmin - 0.5d0) * dwn - wmin) / dwn
      call FCAT_boson_all(378)
  jmax = int(0.5d0 + wmax / dwn)
      call FCAT_boson_all(379)
  test = dmax1(1.5d0 * dabs(emin), 1.5d0 * dabs(emax), 6 * wmax)
      call FCAT_boson_all(380)
  n = 2
      call FCAT_boson_all(381)
  do m = 2, mmax
      call FCAT_boson_all(382)
    n = 2 * n
      call FCAT_boson_all(383)
    wmax = n * dwn
      call FCAT_boson_all(384)
    if (wmax >= test) exit
      call FCAT_boson_all(385)
  enddo
      call FCAT_boson_all(386)
  if (wmax < test) then
      call FCAT_boson_all(387)
    m = mmax
      call FCAT_boson_all(388)
    n = nmax
      call FCAT_boson_all(389)
    wmax = n * dwn
      call FCAT_boson_all(390)
    if (debug) write(*,*) ' +++ n has been fixed at', nmax, ' = 2**', mmax, ' +++'
      call FCAT_boson_all(391)
  endif
      call FCAT_boson_all(392)
  if (debug) then
      call FCAT_boson_all(393)
    write(*,*) ' classical spectrum redefined from 0.0 to', wmax
      call FCAT_boson_all(394)
    write(*,*) ' step size =', dwn, ', ', n, ' (= 2**', m, ') points'
      call FCAT_boson_all(395)
  endif

      call FCAT_boson_all(396)
  allocate (p1(n))
      call FCAT_boson_all(397)
  allocate (p2(n))

      call FCAT_boson_all(398)
  p1 = 0

! *** interpolate pcl on a suitable mesh in (0, wmax)
      call FCAT_boson_all(399)
  i = 1
      call FCAT_boson_all(400)
  do j = jmin, min(jmax, n)
      call FCAT_boson_all(401)
    p1(j) = p(i) + fac * (p(i + 1) - p(i))
      call FCAT_boson_all(402)
    i = i + 1
      call FCAT_boson_all(403)
  enddo

      call FCAT_boson_all(404)
  p2 = p1

! *** characteristic function f(tau)

      call FCAT_boson_all(405)
  call sicot(p1, m, dwn, 1.39d0 * t)
      call FCAT_boson_all(406)
  call sintr(p2, m, dwn)
      call FCAT_boson_all(407)
  h = (pi + pi) / wmax
      call FCAT_boson_all(408)
  if (gauss < 0.0d0 .or. gauss > 1.0d0) then

! *** broaden the spectrum by convoluting the characteristic function
! *** with a user-supplied response function (arbitrary normalization)

      call FCAT_boson_all(409)
    if (debug) write(*,*) '==> switch to a user-supplied instrumental response'
      call FCAT_boson_all(410)
    alfa = dmax1(4 * dwn, width)
      call FCAT_boson_all(411)
    if (alfa > width) then
      call FCAT_boson_all(412)
      if (debug) write(*,*) ' +++ width has been enlarged to', alfa, ' +++'
      call FCAT_boson_all(413)
    endif
      call FCAT_boson_all(414)
    if (alfa < 10 * dwn) then
      call FCAT_boson_all(415)
      if (debug) then
      call FCAT_boson_all(416)
        write(*,*) ' ... poor representation of the response function ...'
      call FCAT_boson_all(417)
        write(*,*) 'the step size (', dwn, ') should be reduced'
      call FCAT_boson_all(418)
      endif
      call FCAT_boson_all(419)
    endif
! make a table of the response function
      call FCAT_boson_all(420)
    r1(1) = respon(0.0d0, alfa)
      call FCAT_boson_all(421)
    r2(1) = r1(1)
      call FCAT_boson_all(422)
    do i = 2, n
      call FCAT_boson_all(423)
      x = (i - 1) * dwn
      call FCAT_boson_all(424)
      r1(i) = respon( x, alfa)
      call FCAT_boson_all(425)
      r2(i) = respon(-x, alfa)
      call FCAT_boson_all(426)
    enddo
! fourier transform it
      call FCAT_boson_all(427)
    call rcffi(r1, r2, -m, dwn)
! normalization of the response function, and convolution
      call FCAT_boson_all(428)
    anorm = r1(1)
      call FCAT_boson_all(429)
    do i = 1, n
      call FCAT_boson_all(430)
      fac = dexp(-p1(i)) / anorm
      call FCAT_boson_all(431)
      cp2 = fac * dcos(p2(i))
      call FCAT_boson_all(432)
      sp2 = fac * dsin(p2(i))
      call FCAT_boson_all(433)
      p1(i) = r1(i) * cp2 + r2(i) * sp2
      call FCAT_boson_all(434)
      p2(i) = r2(i) * cp2 - r1(i) * sp2
      call FCAT_boson_all(435)
    enddo
      call FCAT_boson_all(436)
    alfa = alfa / 2

  else

! *** broaden the spectrum by convoluting the characteristic function by
! *** a weighted sum of a lorentzian and a gaussian response functions

      call FCAT_boson_all(437)
      call FCAT_boson_all(438)
    alfa = dmax1(1.5d0 * dwn, width)
      call FCAT_boson_all(439)
    if (alfa > width) then
      call FCAT_boson_all(440)
      if (debug) write(*,*) ' +++ width has been enlarged to', alfa, ' +++'
      call FCAT_boson_all(441)
    endif
      call FCAT_boson_all(442)
    if (alfa > 0.5d0 * wmax / pi) then
      call FCAT_boson_all(443)
      call FCAT_boson_all_rep()
      stop '*** width is too large, nothing done ***'
      call FCAT_boson_all(444)
    endif
      call FCAT_boson_all(445)
    alfa = 0.5d0 * alfa
      call FCAT_boson_all(446)
    sigma = alfa / 1.66511d0
      call FCAT_boson_all(447)
    a1 = (1.0d0 - asym) / 2 * (1.0d0 - gauss)
      call FCAT_boson_all(448)
    a2 = (1.0d0 + asym) / 2 * (1.0d0 - gauss)
      call FCAT_boson_all(449)
    if (a1 < 0.0d0 .or. a2 < 0.0d0) then
      call FCAT_boson_all(450)
      call FCAT_boson_all_rep()
      stop '*** invalid input : asym should be in (-1, +1) ***'
      call FCAT_boson_all(451)
    endif
      call FCAT_boson_all(452)
    g1 = (1.0d0 - asym) * alfa
      call FCAT_boson_all(453)
    g2 = (1.0d0 + asym) * alfa
      call FCAT_boson_all(454)
    p1(1) = 1.0d0
      call FCAT_boson_all(455)
    p2(1) = 0.0d0
      call FCAT_boson_all(456)
    do i = 2, n
      call FCAT_boson_all(457)
      x = (i - 1) * h
      call FCAT_boson_all(458)
      fr = 0.0d0
      call FCAT_boson_all(459)
      fi = 0.0d0
      call FCAT_boson_all(460)
      if (a1 /= 0.0d0) then
      call FCAT_boson_all(461)
        x1 = g1 * x
      call FCAT_boson_all(462)
        if (x1 <= 100.0d0) then
      call FCAT_boson_all(463)
          fr = a1 * dexp(-x1)
      call FCAT_boson_all(464)
        endif
      call FCAT_boson_all(465)
        if (a1 /= a2) then
      call FCAT_boson_all(466)
          if (x1 <= 1.3d0) then
      call FCAT_boson_all(467)
            fi = a1 * o1(x1)
          else
      call FCAT_boson_all(468)
      call FCAT_boson_all(469)
            fi = a1 * o2(x1)
      call FCAT_boson_all(470)
          endif
      call FCAT_boson_all(471)
        endif
      call FCAT_boson_all(472)
      endif
      call FCAT_boson_all(473)
      if (a2 /= 0.0d0) then
      call FCAT_boson_all(474)
        x2 = g2 * x
      call FCAT_boson_all(475)
        if (x2 <= 100.0d0) then
      call FCAT_boson_all(476)
          fr = fr + a2 * dexp(-x2)
      call FCAT_boson_all(477)
        endif
      call FCAT_boson_all(478)
        if (a2 /= a1) then
      call FCAT_boson_all(479)
          if (x2 <= 1.3d0) then
      call FCAT_boson_all(480)
            fi = fi - a2 * o1(x2)
          else
      call FCAT_boson_all(481)
      call FCAT_boson_all(482)
            fi = fi - a2 * o2(x2)
      call FCAT_boson_all(483)
          endif
      call FCAT_boson_all(484)
        endif
      call FCAT_boson_all(485)
      endif
      call FCAT_boson_all(486)
      if (gauss /= 0.0d0) then
      call FCAT_boson_all(487)
        x3 = sigma * x
      call FCAT_boson_all(488)
        if (x3 <= 10.0d0) then
      call FCAT_boson_all(489)
          fr = fr + gauss * dexp(-x3**2)
      call FCAT_boson_all(490)
        endif
      call FCAT_boson_all(491)
      endif
      call FCAT_boson_all(492)
      fi = fi / pi
      call FCAT_boson_all(493)
      fac = dexp(-p1(i))
      call FCAT_boson_all(494)
      cp2 = fac * dcos(p2(i))
      call FCAT_boson_all(495)
      sp2 = fac * dsin(p2(i))
      call FCAT_boson_all(496)
      p1(i) = fr * cp2 + fi * sp2
      call FCAT_boson_all(497)
      p2(i) = fi * cp2 - fr * sp2
      call FCAT_boson_all(498)
    enddo
      call FCAT_boson_all(499)
    if (dabs(fi) > 1.0d-03) then
      call FCAT_boson_all(500)
      if (debug) then
      call FCAT_boson_all(501)
        write(*,*) ' ... poor representation of the response function ...'
      call FCAT_boson_all(502)
        write(*,*) 'the step size (', dwn, ') should be reduced'
      call FCAT_boson_all(503)
      endif
      call FCAT_boson_all(504)
    endif
      call FCAT_boson_all(505)
  endif

! *** full eels spectrum

      call FCAT_boson_all(506)
   call rcffi(p1, p2, m, h)

! *** output

      call FCAT_boson_all(507)
  istep = max(int(alfa / dwn / 10), 1)
      call FCAT_boson_all(508)
  nout = 0
      call FCAT_boson_all(509)
  if (emin <= 0.0d0) then
      call FCAT_boson_all(510)
    imin = n - int(dabs(emin) / dwn)
      call FCAT_boson_all(511)
    if ((imin - n) * dwn < emin) then
      call FCAT_boson_all(512)
      imin = imin + 1
      call FCAT_boson_all(513)
    endif
      call FCAT_boson_all(514)
    do i = imin, n
      call FCAT_boson_all(515)
      j = n - i
      call FCAT_boson_all(516)
      if (mod(j, istep) > 0) cycle
      call FCAT_boson_all(517)
      x = -j * dwn
      call FCAT_boson_all(518)
      if (x > emax) exit
      call FCAT_boson_all(519)
      nout = nout + 1
      call FCAT_boson_all(520)
      xout(nout)= x
      call FCAT_boson_all(521)
      yout(nout)= p2(j + 1)
      call FCAT_boson_all(522)
    enddo
      call FCAT_boson_all(523)
  endif
      call FCAT_boson_all(524)
  if (x <= emax) then
      call FCAT_boson_all(525)
    if (emax >= dwn) then
      call FCAT_boson_all(526)
      imax = int(emax / dwn) + 1
      call FCAT_boson_all(527)
      if ((imax - 1) * dwn > emax) then
      call FCAT_boson_all(528)
        imax = imax - 1
      call FCAT_boson_all(529)
      endif
      call FCAT_boson_all(530)
      if (imax >= 2) then
      call FCAT_boson_all(531)
        do i = 2, imax
      call FCAT_boson_all(532)
          if (mod(i - 1, istep) > 0) cycle
      call FCAT_boson_all(533)
          x = (i - 1) * dwn
      call FCAT_boson_all(534)
          if (x < emin) cycle
      call FCAT_boson_all(535)
          nout = nout + 1
      call FCAT_boson_all(536)
          xout(nout) = x
      call FCAT_boson_all(537)
          yout(nout) = p1(i)
      call FCAT_boson_all(538)
        enddo
      call FCAT_boson_all(539)
      endif
      call FCAT_boson_all(540)
    endif
      call FCAT_boson_all(541)
  endif
      call FCAT_boson_all(542)
  if (debug) write(*,*) nout, ' values, step size =', istep * dwn

! *** analyze the spectrum

      call FCAT_boson_all(543)
  if (debug) write(*,*) ' peak location    amplitude'
      call FCAT_boson_all(544)
  wn = 0.0d0
      call FCAT_boson_all(545)
  fm = p1(1)
      call FCAT_boson_all(546)
  fp = p2(1)
      call FCAT_boson_all(547)
  if (p2(2) < fp .and. p1(2) < fp) then
      call FCAT_boson_all(548)
    if (debug) write(*, '(f13.2, e13.4)') wn, fp
      call FCAT_boson_all(549)
  endif
      call FCAT_boson_all(550)
  fac = 5.0d-06 * fp * dwn
      call FCAT_boson_all(551)
  imax = 2 + int(dmax1(dabs(emin), dabs(emax)) / dwn)
      call FCAT_boson_all(552)
  do i = 2, imax
      call FCAT_boson_all(553)
    fm0 = fm1
      call FCAT_boson_all(554)
    fp0 = fp1
      call FCAT_boson_all(555)
    fm1 = fm
      call FCAT_boson_all(556)
    fp1 = fp
      call FCAT_boson_all(557)
    fm = p2(i)
      call FCAT_boson_all(558)
    fp = p1(i)
      call FCAT_boson_all(559)
    if (i == 2) cycle
      call FCAT_boson_all(560)
    picm = .false.
      call FCAT_boson_all(561)
    picp = .false.
      call FCAT_boson_all(562)
    wn = (i - 1) * dwn
      call FCAT_boson_all(563)
    if ((fm1 >= fm0) .and. (fm1 >= fm)) then
      call FCAT_boson_all(564)
      a = (fm1 - fm0) + (fm1 - fm)
      call FCAT_boson_all(565)
      if (a >= fac) then
      call FCAT_boson_all(566)
        b = 0.5d0 * ((fm1 - fm0) + 3 * (fm1 - fm))
      call FCAT_boson_all(567)
        u = b / a
      call FCAT_boson_all(568)
        wmpic = -wn + u * dwn
      call FCAT_boson_all(569)
        fmpic = fm + 0.5d0 * b * u
      call FCAT_boson_all(570)
        picm = .true.
      call FCAT_boson_all(571)
      endif
      call FCAT_boson_all(572)
    endif
      call FCAT_boson_all(573)
    if ((fp1 >= fp0) .and. (fp1 >= fp)) then
      call FCAT_boson_all(574)
      a = (fp1 - fp0) + (fp1 - fp)
      call FCAT_boson_all(575)
      if (a >= fac) then
      call FCAT_boson_all(576)
        b = 0.5d0 * ((fp1 - fp0) + 3 * (fp1 - fp))
      call FCAT_boson_all(577)
        u = b / a
      call FCAT_boson_all(578)
        wppic = wn - u * dwn
      call FCAT_boson_all(579)
        fppic = fp + 0.5d0 * b * u
      call FCAT_boson_all(580)
        picp = .true.
      call FCAT_boson_all(581)
        if (picp) then
      call FCAT_boson_all(582)
          if (picm) then
      call FCAT_boson_all(583)
            if (debug) write(*, '(2(f13.2, e13.4, 5x))') wppic, fppic, wmpic, fmpic
          else
      call FCAT_boson_all(584)
      call FCAT_boson_all(585)
            if (debug) write(*, '(f13.2, e13.4)') wppic, fppic
      call FCAT_boson_all(586)
          endif
      call FCAT_boson_all(587)
        endif
      call FCAT_boson_all(588)
      endif
      call FCAT_boson_all(589)
    endif
      call FCAT_boson_all(590)
    if (picm .and. (.not. picp)) then
      call FCAT_boson_all(591)
      if (debug) write(*, '(33x, f13.2, e13.4)') wmpic, fmpic
      call FCAT_boson_all(592)
    endif
      call FCAT_boson_all(593)
  enddo
      call FCAT_boson_all(594)
  deallocate (p1)
      call FCAT_boson_all(595)
  deallocate (p2)
      call FCAT_boson_all(596)
  return
end subroutine doboson

double precision function o1(u)

  implicit none

  double precision, intent(in) :: u
  double precision :: u2

      call FCAT_boson_all(597)
  u2 = u**2
      call FCAT_boson_all(598)
  o1 = -dsinh(u) * dlog(u2) + u * ((0.03114d0 * u2 + 0.41666d0) * u2 + 0.84557d0)

      call FCAT_boson_all(599)
  return
end function o1

double precision function o2(u)

  implicit none

  double precision, intent(in) :: u
  double precision :: u2

      call FCAT_boson_all(600)
  u2 = 1 / u**2
      call FCAT_boson_all(601)
  o2 = (((202.91d0 * u2 + 932.21d0) * u2 + 41.740d0) * u2 + 2.0d0) /  &
       (((540.88d0 * u2 + 345.67d0) * u2 + 18.961d0) * u2 + 1.0d0) / u

      call FCAT_boson_all(602)
  return
end function o2
       module FCAT_boson_all_mod
        double precision,dimension (603):: &
     &   FCAT_boson_all_counter = 0
        integer :: FCAT_boson_all_nline = 602
       end module FCAT_boson_all_mod
       subroutine FCAT_boson_all(n)
         use FCAT_boson_all_mod
         integer :: n
         FCAT_boson_all_counter(n) = &
     &    FCAT_boson_all_counter(n)  + 1
         if (FCAT_boson_all_counter(n) == 1) then
           write(*,"(a,i10)") "FCAT_boson_all_",n
         endif
       end subroutine FCAT_boson_all
       subroutine FCAT_boson_all_rep()
          use FCAT_boson_all_mod
          integer :: i
          do i = 1, FCAT_boson_all_nline
             write(*,"(a,i10,i10)") &
     &      "FCAT_boson_all_count",i, &
     &      int(FCAT_boson_all_counter(i)+0.1)
          end do
       end subroutine FCAT_boson_all_rep
